#ifndef YERBACON_PARSECOMPONENTS_HPP
#define YERBACON_PARSECOMPONENTS_HPP

#include <iostream>
#include <utility>
#include <vector>
using namespace std;

#include "../lex.hpp"

struct ParseComponent {
    virtual ~ParseComponent() = default;
};

class ParseTree: public ParseComponent {
protected:
    mutable vector<ParseComponent> subComponents;
public:
    size_t getCompCount() const { return subComponents.size(); }
    auto& getComponents() const { return subComponents; }
    void add(const ParseComponent& component) const {
        subComponents.push_back(component);
    }; void addAll(const vector<ParseComponent>& components) const {
        for (const auto& comp: components) add(comp);
    }; void operator<<(const ParseComponent& component) const { add(component); }
    explicit ParseTree(const ParseComponent& element): subComponents() { add(element); }
    ParseTree(const initializer_list<ParseComponent>& elements): subComponents() { addAll(elements); }
    ParseTree(): subComponents() {};
};

namespace StandardComponents {
    struct [[deprecated]] Expression: ParseComponent {};
    struct Define: ParseComponent {
        const bool final;
        explicit Define(const bool& isFinal): final(isFinal) {}
        // Shorthand for Define(false)
        Define(): Define(false) {}
    };
    struct Constructor: ParseComponent {};
    struct ClassBody: ParseComponent {};
    struct Class: ParseComponent {
        string bullshitText;
        Constructor constructor;
        explicit Class(string_view text): bullshitText(text) {}
    };
    namespace types {
        struct String: ParseComponent {
            [[maybe_unused]] const char* content;
            explicit String(const char* string): content(string) {}
        };
    }
}

#endif //YERBACON_PARSECOMPONENTS_HPP